#!/usr/bin/perl

#
#    Script for finding usless accepted outputs for a set of ACL2 versions
#    Copyright (C) 2009 Antonio García Domínguez
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# Usage:
#
# 1. Run "t/regression.t" one time per each version of ACL2, switching
# the 'acl2' binary available through the PATH variable to the proper
# version. For instance, this wrapping script can be used:
#
# #!/bin/sh
# if [ -n "$ACL2V" ]; then
#    acl2-"$ACL2V"
# else
#    exec ~/bin/acl2-3.4
# fi
#
# Save the output from these tests to a set of files or just a single
# file.
#
# 2. Run this script passing as arguments all the files previously
# produced.

use strict;
use warnings;
use utf8;
use File::Basename;

my @lines           = <>;
my @used_files      = grep {$_} (map {/\(matches ([^,]+)(, [^,]+)*\)/} @lines);
my %hash_used_files = map {(basename $_) => 1} @used_files;
my @uniq_used_files = sort (keys %hash_used_files);

my @paths_existing  = <t/expectedOutputs/*.xml.test*>;
my @existing_files  = map {basename $_} @paths_existing;

my $used_count       = scalar @uniq_used_files;
my $existing_count   = scalar @existing_files;
my $counter_used     = 0;
my $counter_existing = 0;

while ($counter_used < $used_count &&
       $counter_existing < $existing_count) {

    my $current_used     = $uniq_used_files[$counter_used];
    my $current_existing = $existing_files[$counter_existing];

    if ($current_used lt $current_existing) {
        die "Used though it does not exist: $current_used\n";
        $counter_used++;
    }
    elsif ($current_existing lt $current_used) {
        print "Not used: $current_existing\n";
        $counter_existing++;
    }
    else {
        $counter_used++;
        $counter_existing++;
    }
}
if ($counter_used < $used_count) {
    my $current_used     = $uniq_used_files[$counter_used];
    die "Used though it does not exist: $current_used\n";
}
while ($counter_existing < $existing_count) {
    my $current_existing = $existing_files[$counter_used];
    print "Not used: $current_existing\n";
    $counter_existing++;
}
