package ACL2::Lisp::Symbol;
use base 'ACL2::Lisp::BaseNode';

#
#  AST node for symbols
#  Copyright (C) 2009 Antonio García Domínguez
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

sub new
{
    my ($pkg, $value, $source) = @_;
    return ACL2::Lisp::BaseNode::new(
        $pkg, $value, (defined($source) ? $source : uc $value));
}

=head2 escaped_value($self)

Returns the value of this symbol, quoted if required, by adding |
around the value and using single escapes on any remaining | and \
characters. Case is preserved, and multiple escape sequences are
preferred.

=cut

sub escaped_value
{
    my $original_value = $_[0]->value;

    # For some reason, the usual "use Module qw(methods)" syntax at
    # the start of the file doesn't do anything (probably due to some
    # interaction between the initial import at BEGIN and "use
    # base"). This works, however.
    require ACL2::Lisp::Parser;
    ACL2::Lisp::Parser->import(qw(is_potential_number is_alpha_lower
                                  is_whitespace is_macro is_single_escape
                                  is_multiple_escape));

    if (is_potential_number($original_value)) {
        return "|${original_value}|";
    }

    foreach (split //, $original_value) {
        (is_alpha_lower($_) || is_whitespace($_) ||
         is_macro($_) ||
         is_single_escape($_) || is_multiple_escape($_))
            && do {
                my $escaped_value = $original_value;
                $escaped_value   =~ s/\\/\\\\/g;
                $escaped_value   =~ s/\|/\\\|/g;
                return "|${escaped_value}|";
        };
    }

    return $original_value;
}

1;
