package ACL2::Proceso::Simplificacion;
use base ACL2::Proceso;

=head1 NAME

ACL2::Proceso::Simplificacion - simplificacion por reescritura de terminos

=head1 FUNCTIONS

=cut

#
#    Clase Simplificación: tipo de proceso de demostración de ACL2
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use vars;
use utf8;

use ACL2::Orden;
use ACL2::Proceso::Fabrica;

# Registro
BEGIN {
ACL2::Proceso::Fabrica->registrar
    (
     __PACKAGE__, 
     sub { 
	 return ACL2::Proceso::Simplificacion->_identificar(@_);
     }
    );
}

sub escribir_xml_cabecera
{
    my ($this, $writer) = @_;
    if (defined($this->{producto})) {
	$writer->startTag(ACL2::Tags::SIMPL_TAG,
			  "res" => $this->{producto});
    } else {
	$writer->startTag(ACL2::Tags::SIMPL_TAG);
    }

    ACL2::Proceso->volcar_reglas($writer, $this->{reglas});
    ACL2::Orden->salida_parrafos($writer, $this->{salida});
}

sub escribir_xml_final
{
    my ($this, $writer) = @_;
    $writer->endTag(ACL2::Tags::SIMPL_TAG);
}

=head2 _identificar

Uso: ME<eacute>todo estE<aacute>tico privado.

Responsabilidad: Realizar la identificaciE<oacute>n de una SimplificaciE<oacute>n.

ParE<aacute>metros: salida - salida de la meta.

Devuelve: lista de argumentos para new, o undef si no coincide.

=cut

sub _identificar {
    caller eq __PACKAGE__ or die;
    my ($paquete, $salida) = @_;
    
    my %res;
    
    $salida =~ 
	/
	(?-x:^\s*But simplification reduces this to T, using ([^\.]+)\.)
	|
	(?-x:^\s*But we reduce the conjecture to T, by ([^\.]+)\.)
	|
	(?-x:^\s*By the simple (.+?) we reduce the conjecture to\b)
	|
	(?-x:^\s*This simplifies, using (.*?), to\b)
	|
	(?-x:^\s*By (.*?) we reduce the conjecture to\b)
	/ox or return undef;
    
    $res{producto} = (defined($1||$2) ? "T" : undef);
    $res{reglas}   = ACL2::Proceso->obtener_reglas($1 || $2 || $3 || $4 || $5);
    $res{salida}   = $salida;
    
    return \%res;
}

1;
