#!perl -T
# -*- perl -*-

#
# Test that Lisp parsing errors include line numbers
# Antonio García Domínguez
# $Id$
#

use strict;
use warnings;
use utf8;
use Test::More;

use ACL2::Lisp::Parser qw(read_lisp_program);

my @error_lines = (
    "º"         => 1,
    "((\nx"     => 2,
    "#|\na\n\n" => 4,
    "a\n#"      => 2,
    "#x\n"      => 1,
    "|hi\n\nthere"  => 3,
    "\"str\nhey\nthere" => 3,
    "x\n,invalid"   => 2,
    "#2|\nfoo\nbar" => 1,
    "a\nb\nc\n\\"   => 4,
    );

plan tests => scalar @error_lines;

while (@error_lines > 0) {
    my ($source, $line) = splice @error_lines, 0, 2;

    eval {
        read_lisp_program(\$source);
    };
    my $msg = $@;

    ok(length($msg) > 0,
       "An error should have been reported while parsing '$source'");
    like($msg, qr/\b$line\b/,
         "Error '$msg' should contain the expected line number ($line).");
}
