#!perl -T

#
# Test that all .pm sources have the GPL license bit
# Antonio García Domínguez
# $Id$
#
use strict;
use warnings;
use Test::More;

# Build up the list of all .pm source files inside lib
my @sources = get_sources();

# Avisamos a Test::More del número de pruebas a realizar
plan tests => scalar @sources;

# Expresión regular para comprobar que se tiene el texto de la GPL

my $re_gpl = qr!\s*#?\s*\QThis program is free software; you can redistribute it and/or modify\E
\s*#?\s*\Qit under the terms of the GNU General Public License as published by\E
\s*#?\s*\Qthe Free Software Foundation; either version 2 of the License, or\E
\s*#?\s*\Q(at your option) any later version.\E
\s*#?\s*
\s*#?\s*\QThis program is distributed in the hope that it will be useful,\E
\s*#?\s*\Qbut WITHOUT ANY WARRANTY; without even the implied warranty of\E
\s*#?\s*\QMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\E
\s*#?\s*\QGNU General Public License for more details.\E
\s*#?\s*
\s*#?\s*\QYou should have received a copy of the GNU General Public License\E
\s*#?\s*\Qalong with this program; if not, write to the Free Software\E
\s*#?\s*\QFoundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\E!;

foreach my $source (@sources) {
    my $fh;
    open($fh, "<", $source)
	or die "Couldn't open $source: $!";
    my @lines = <$fh>;
    my $text  = join "\n", @lines;
    ok($text =~ $re_gpl, "$source should contain the boilerplate GPL text");
}

## UTILITY FUNCTIONS

sub get_sources
{
    my @sources  = ();
    my @dirQueue = ("lib");
    while (@dirQueue) {
	my $dir   = shift @dirQueue;
	
	local *DIR;
	opendir(DIR, $dir)
	    or die "Couldn't open the '$dir' directory: $!";
	foreach my $entry (readdir(DIR)) {
	    next if $entry =~ /^\./;
	    my $fullpath = "$dir/$entry";
	    
	    push @dirQueue, $fullpath if -d $fullpath;
	    push @sources, $fullpath if -f $fullpath && $entry =~ /\.pm$/;
	}
	
	closedir(DIR);
    }
    return @sources;
}
